@@ -2,11 +2,11 @@
 /* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */
 
 /**
- * Zend_Db wrapper
+ * Zend Db wrapper
  *
  * PHP version 5
  *
- * Copyright © 2011-2013 The Galette Team
+ * Copyright © 2011-2014 The Galette Team
  *
  * This file is part of Galette (http://galette.tuxfamily.org).
  *
@@ -27,7 +27,7 @@
  * @package   Galette
  *
  * @author    Johan Cwiklinski <johan@x-tnd.be>
- * @copyright 2011-2013 The Galette Team
+ * @copyright 2011-2014 The Galette Team
  * @license   http://www.gnu.org/licenses/gpl-3.0.html GPL License 3.0 or (at your option) any later version
  * @version   SVN: $Id$
  * @link      http://galette.tuxfamily.org
@@ -37,20 +37,22 @@
 namespace Galette\Core;
 
 use Analog\Analog as Analog;
+use Zend\Db\Adapter\Adapter;
+use Zend\Db\Sql\Sql;
 
 /**
- * Zend_Db wrapper
+ * Zend Db wrapper
  *
  * @category  Core
  * @name      Db
  * @package   Galette
  * @author    Johan Cwiklinski <johan@x-tnd.be>
- * @copyright 2011-2013 The Galette Team
+ * @copyright 2011-2014 The Galette Team
  * @license   http://www.gnu.org/licenses/gpl-3.0.html GPL License 3.0 or (at your option) any later version
- * @link      http://framework.zend.com/apidoc/core/_Db.html#\Zend_Db
+ * @link      http://framework.zend.com/apidoc/2.2/namespaces/Zend.Db.html
  * @since     Available since 0.7dev - 2011-07-27
  */
-class Db extends \Zend_Db
+class Db
 {
     private $_persistent;
     private $_dsn_array;
@@ -59,6 +61,7 @@ class Db extends \Zend_Db
     private $_db;
     private $_error;
     private $_type_db;
+    private $_sql;
 
     const MYSQL = 'mysql';
     const PGSQL = 'pgsql';
@@ -111,37 +114,31 @@ class Db extends \Zend_Db
             $this->_type_db = $_type_db;
             if ($_type_db != self::SQLITE) {
                 $_options = array(
-                        'host'     => $_host_db,
-                        'port'     => $_port_db,
-                        'username' => $_user_db,
-                        'password' => $_pwd_db,
-                        'dbname'   => $_name_db
-                    );
+                    'driver'   => $_type,
+                    'hostname' => $_host_db,
+                    'port'     => $_port_db,
+                    'username' => $_user_db,
+                    'password' => $_pwd_db,
+                    'database' => $_name_db
+                );
+                if ($_type_db === self::MYSQL) {
+                    $_options['charset'] = 'utf8';
+                }
             } else {
                 $_options = array(
-                        'dbname'   => GALETTE_SQLITE_PATH,
-                    );
+                    'driver'   => $_type,
+                    'database' => GALETTE_SQLITE_PATH,
+                );
             }
 
-            $this->_db = \Zend_Db::factory(
-                $_type,
-                $_options
-            );
-            $this->_db->getConnection();
-            $this->_db->setFetchMode(\Zend_Db::FETCH_OBJ);
+            $this->_db = new Adapter($_options);
+            $this->_db->getDriver()->getConnection()->connect();
+            $this->_sql = new Sql($this->_db);
+
             Analog::log(
                 '[Db] Database connection was successfull!',
                 Analog::DEBUG
             );
-        } catch (\Zend_Db_Adapter_Exception $e) {
-            // perhaps a failed login credential, or perhaps the RDBMS is not running
-            $ce = $e->getChainedException();
-            Analog::log(
-                '[Db] No connexion (' . $ce->getCode() . '|' .
-                $ce->getMessage() . ')',
-                Analog::ALERT
-            );
-            throw $e;
         } catch (\Exception $e) {
             // perhaps factory() failed to load the specified Adapter class
             Analog::log(
@@ -154,11 +151,36 @@ class Db extends \Zend_Db
     }
 
     /**
-     * Default class destructor
+     * Retrieve current database version
+     *
+     * @return float
+     *
+     * @throw LogicException
      */
-    function __destruct()
+    public function getDbVersion()
     {
-        $this->db->closeConnection();
+        try {
+            $select = $this->select('database');
+            $select->columns(
+                array('version')
+            )->limit(1);
+
+            $results = $this->execute($select);
+            $result = $results->current();
+            return number_format(
+                $result->version,
+                3,
+                '.',
+                ''
+            );
+        } catch ( \Exception $e ) {
+            Analog::log(
+                'Cannot check database version: ' . $e->getMessage(),
+                Analog::ERROR
+            );
+            throw new \LogicException('Cannot check database version');
+        }
+
     }
 
     /**
@@ -175,20 +197,10 @@ class Db extends \Zend_Db
             );
             return true;
         }
+
         try {
-            $select = new \Zend_Db_Select($this->db);
-            $select->from(
-                PREFIX_DB . 'database',
-                array('version')
-            )->limit(1);
-            $sql = $select->__toString();
-            $res = $select->query()->fetch();
-            return $res->version === GALETTE_DB_VERSION;
-        } catch ( \Exception $e ) {
-            Analog::log(
-                'Cannot check database version: ' . $e->getMessage(),
-                Analog::ERROR
-            );
+            return $this->getDbVersion() === GALETTE_DB_VERSION;
+        } catch ( \LogicException $e ) {
             return false;
         }
     }
@@ -202,58 +214,28 @@ class Db extends \Zend_Db
      */
     public function selectAll($table)
     {
-        return $this->_db->fetchAll('SELECT * FROM ' . $table);
+        return $this->_db->query(
+            'SELECT * FROM ' . PREFIX_DB . $table,
+            Adapter::QUERY_MODE_EXECUTE
+        );
     }
 
     /**
-     * List updates scripts from given path
+     * Test if database can be contacted. Mostly used for installation
      *
-     * @param string $path    Scripts path
-     * @param string $db_type Database type
-     * @param string $version Current version
+     * @param string $type db type
+     * @param string $user database's user
+     * @param string $pass password for the user
+     * @param string $host which host we want to connect to
+     * @param string $port which tcp port we want to connect to
+     * @param string $db   database name
      *
-     * @return array
+     * @return true|array true if connection was successfull,
+     *                    an array with some infos otherwise
      */
-    public static function getUpdateScripts($path, $db_type = 'mysql', $version = null)
-    {
-        $dh = opendir($path . '/sql');
-        $update_scripts = array();
-        if ( $dh !== false ) {
-            while ( ($file = readdir($dh)) !== false ) {
-                if ( preg_match("/upgrade-to-(.*)-" . $db_type . ".sql/", $file, $ver) ) {
-                    if ( $version === null ) {
-                        $update_scripts[] = $ver[1];
-                    } else {
-                        if ( $version <= $ver[1] ) {
-                            $update_scripts[$ver[1]] = $file;
-                        }
-                    }
-                }
-            }
-            closedir($dh);
-            if ( $version === null ) {
-                asort($update_scripts);
-            } else {
-                ksort($update_scripts);
-            }
-        }
-        return $update_scripts;
-    }
-
-    /**
-    * Test if database can be contacted. Mostly used for installation
-    *
-    * @param string $type db type
-    * @param string $user database's user
-    * @param string $pass password for the user
-    * @param string $host which host we want to connect to
-    * @param string $port which tcp port we want to connect to
-    * @param string $db   database name
-    *
-    * @return true|array true if connection was successfull, an array with some infos otherwise
-    */
-    public static function testConnectivity($type, $user = null, $pass = null, $host = null, $port = null, $db = null)
-    {
+    public static function testConnectivity(
+        $type, $user = null, $pass = null, $host = null, $port = null, $db = null
+    ) {
         $_type = null;
         try {
             if ( $type === self::MYSQL ) {
@@ -266,51 +248,35 @@ class Db extends \Zend_Db
                 throw new \Exception;
             }
 
+
             if ($type != self::SQLITE) {
                 $_options = array(
-                        'host'     => $host,
-                        'port'     => $port,
-                        'username' => $user,
-                        'password' => $pass,
-                        'dbname'   => $db
-                    );
+                    'driver'   => $_type,
+                    'hostname' => $host,
+                    'port'     => $port,
+                    'username' => $user,
+                    'password' => $pass,
+                    'database' => $db
+                );
             } else {
                 $_options = array(
-                        'dbname'   => GALETTE_SQLITE_PATH,
-                    );
+                    'driver'   => $_type,
+                    'databse'   => GALETTE_SQLITE_PATH,
+                );
             }
 
-            $_db = \Zend_Db::factory(
-                $_type,
-                $_options
-            );
-            $_db->getConnection();
-            $_db->setFetchMode(\Zend_Db::FETCH_OBJ);
-            $_db->closeConnection();
+            $_db = new Adapter($_options);
+            $_db->getDriver()->getConnection()->connect();
+
             Analog::log(
                 '[' . __METHOD__ . '] Database connection was successfull!',
                 Analog::DEBUG
             );
             return true;
-        } catch (\Zend_Db_Adapter_Exception $e) {
-            // perhaps a failed login credential, or perhaps the RDBMS is not running
-            $_code = $e->getCode();
-            $_msg = $e->getMessage();
-            $ce = $e->getChainedException();
-            if ( $ce ) {
-                $_code = $ce->getCode();
-                $_msg = $ce->getMessage();
-            }
-            Analog::log(
-                '[' . __METHOD__ . '] No connexion (' . $_code . '|' .
-                $_msg . ')',
-                Analog::ALERT
-            );
-            return $e;
         } catch (\Exception $e) {
-            // perhaps factory() failed to load the specified Adapter class
+            // perhaps failed to load the specified Adapter class
             Analog::log(
-                '[' . __METHOD__ . '] Error (' . $e->getCode() . '|' .
+                '[' . __METHOD__ . '] Connection error (' . $e->getCode() . '|' .
                 $e->getMessage() . ')',
                 Analog::ALERT
             );
@@ -320,10 +286,10 @@ class Db extends \Zend_Db
     }
 
     /**
-    * Drop test table if it exists, so we can make all checks.
-    *
-    * @return void
-    */
+     * Drop test table if it exists, so we can make all checks.
+     *
+     * @return void
+     */
     public function dropTestTable()
     {
         try {
@@ -338,14 +304,14 @@ class Db extends \Zend_Db
     }
 
     /**
-    * Checks GRANT access for install time
-    *
-    * @param char $mode are we at install time (i) or update time (u) ?
-    *
-    * @return array containing each test. Each array entry could
-    *           be either true or contains an exception of false if test did not
-    *           ran.
-    */
+     * Checks GRANT access for install time
+     *
+     * @param char $mode are we at install time (i) or update time (u) ?
+     *
+     * @return array containing each test. Each array entry could
+     *           be either true or contains an exception of false if test did not
+     *           ran.
+     */
     public function grantCheck($mode = 'i')
     {
         Analog::log(
@@ -371,7 +337,7 @@ class Db extends \Zend_Db
                 test_id INTEGER NOT NULL,
                 test_text VARCHAR(20)
             )';
-            $this->_db->getConnection()->exec($sql);
+            $this->_db->query($sql, Adapter::QUERY_MODE_EXECUTE);
             $results['create'] = true;
         } catch (\Exception $e) {
             Analog::log('Cannot CREATE TABLE', Analog::WARNING);
@@ -386,7 +352,7 @@ class Db extends \Zend_Db
                 //can Galette ALTER tables? (only for update mode)
                 try {
                     $sql = 'ALTER TABLE galette_test ALTER test_text SET DEFAULT \'nothing\'';
-                    $this->_db->getConnection()->exec($sql);
+                    $this->_db->query($sql, Adapter::QUERY_MODE_EXECUTE);
                     $results['alter'] = true;
                 } catch (\Exception $e) {
                     Analog::log(
@@ -403,11 +369,12 @@ class Db extends \Zend_Db
                 'test_text'    => 'a simple text'
             );
             try {
-                $res = $this->_db->insert(
-                    'galette_test',
-                    $values
-                );
-                if ( $res === 1 ) {
+                $insert = $this->_sql->insert('galette_test');
+                $insert->values($values);
+
+                $res = $this->execute($insert);
+
+                if ( $res->count() === 1 ) {
                     $results['insert'] = true;
                 } else {
                     throw new \Exception('No row inserted!');
@@ -429,12 +396,12 @@ class Db extends \Zend_Db
                     'test_text' => 'another simple text'
                 );
                 try {
-                    $res = $this->_db->update(
-                        'galette_test',
-                        $values,
-                        array('test_id = ?' => 1)
+                    $update = $this->_sql->update('galette_test');
+                    $update->set($values)->where(
+                        array('test_id' => 1)
                     );
-                    if ( $res === 1 ) {
+                    $res = $this->execute($update);
+                    if ( $res->count() === 1 ) {
                         $results['update'] = true;
                     } else {
                         throw new \Exception('No row updated!');
@@ -449,11 +416,21 @@ class Db extends \Zend_Db
 
                 //can Galette SELECT records ?
                 try {
-                    $select = new \Zend_Db_Select($this->_db);
-                    $select->from('galette_test')
-                        ->where('test_id = ?', 1);
-                    $res = $select->query()->fetchAll();
-                    if ( count($res) === 1 ) {
+                    //For reasons I do not understand, count() always return 0
+                    //with SQLITE but current() give expected result.
+                    //Just executing the query will make DROP test fail
+                    //because database is locked...
+                    $pass = false;
+                    if ( $this->isSQLite() ) {
+                        $pass = true;
+                    } else {
+                        $select = $this->_sql->select('galette_test');
+                        $select->where('test_id = 1');
+                        $res = $this->execute($select);
+                        $pass = $res->count() === 1;
+                    }
+
+                    if ( $pass ) {
                         $results['select'] = true;
                     } else {
                         throw new \Exception('Select is empty!');
@@ -468,10 +445,9 @@ class Db extends \Zend_Db
 
                 //can Galette DELETE records ?
                 try {
-                    $this->_db->delete(
-                        'galette_test',
-                        array('test_id = ?' => 1)
-                    );
+                    $delete = $this->_sql->delete('galette_test');
+                    $delete->where(array('test_id' => 1));
+                    $this->execute($delete);
                     $results['delete'] = true;
                 } catch (\Exception $e) {
                     Analog::log(
@@ -485,7 +461,7 @@ class Db extends \Zend_Db
             //can Galette DROP tables ?
             try {
                 $sql = 'DROP TABLE galette_test';
-                $this->_db->getConnection()->exec($sql);
+                $this->_db->query($sql, Adapter::QUERY_MODE_EXECUTE);
                 $results['drop'] = true;
             } catch (\Exception $e) {
                 Analog::log(
@@ -508,7 +484,8 @@ class Db extends \Zend_Db
      */
     public function getTables($prefix = null)
     {
-        $tmp_tables_list = $this->_db->listTables();
+        $metadata = new \Zend\Db\Metadata\Metadata($this->_db);
+        $tmp_tables_list = $metadata->getTableNames();
 
         if ( $prefix === null ) {
             $prefix = PREFIX_DB;
@@ -525,13 +502,27 @@ class Db extends \Zend_Db
     }
 
     /**
-    * Converts recursively database to UTF-8
-    *
-    * @param string  $prefix       Specified table prefix
-    * @param boolean $content_only Proceed only content (no table conversion)
-    *
-    * @return void
-    */
+     * Get columns for a specified table
+     *
+     * @param string $table Table name
+     *
+     * @return array
+     */
+    public function getColumns($table)
+    {
+        $metadata = new \Zend\Db\Metadata\Metadata($this->_db);
+        $table = $metadata->getTable(PREFIX_DB . $table);
+        return $table->getColumns();
+    }
+
+    /**
+     * Converts recursively database to UTF-8
+     *
+     * @param string  $prefix       Specified table prefix
+     * @param boolean $content_only Proceed only content (no table conversion)
+     *
+     * @return void
+     */
     public function convertToUTF($prefix = null, $content_only = false)
     {
 
@@ -540,7 +531,7 @@ class Db extends \Zend_Db
         }
 
         try {
-            $this->_db->beginTransaction();
+            $this->connection->beginTransaction();
 
             $tables = $this->getTables($prefix);
 
@@ -552,7 +543,11 @@ class Db extends \Zend_Db
                     $query = 'ALTER TABLE ' . $table .
                         ' CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci';
 
-                    $this->_db->getConnection()->exec($query);
+                    $this->_db->query(
+                        $query,
+                        Adapter::QUERY_MODE_EXECUTE
+                    );
+
                     Analog::log(
                         'Charset successfully changed for table `' . $table .'`',
                         Analog::DEBUG
@@ -564,9 +559,9 @@ class Db extends \Zend_Db
                     $this->_convertContentToUTF($prefix, $table);
                 }
             }
-            $this->_db->commit();
+            $this->connection->commit();
         } catch (\Exception $e) {
-            $this->_db->rollBack();
+            $this->connection->rollBack();
             Analog::log(
                 'An error occured while converting to utf table ' .
                 $table . ' (' . $e->getMessage() . ')',
@@ -576,20 +571,22 @@ class Db extends \Zend_Db
     }
 
     /**
-    * Converts dtabase content to UTF-8
-    *
-    * @param string $prefix Specified table prefix
-    * @param string $table  the table we want to convert datas from
-    *
-    * @return void
-    */
+     * Converts dtabase content to UTF-8
+     *
+     * @param string $prefix Specified table prefix
+     * @param string $table  the table we want to convert datas from
+     *
+     * @return void
+     */
     private function _convertContentToUTF($prefix, $table)
     {
 
         try {
             $query = 'SET NAMES latin1';
-            $this->_db->getConnection()->exec($query);
-
+            $this->_db->query(
+                $query,
+                Adapter::QUERY_MODE_EXECUTE
+            );
         }catch (\Exception $e) {
             Analog::log(
                 'Cannot SET NAMES on table `' . $table . '`. ' .
@@ -599,18 +596,15 @@ class Db extends \Zend_Db
         }
 
         try {
-            $select = new \Zend_Db_Select($this->_db);
-            $select->from($table);
-
-            $result = $select->query();
-
-            $descr = $this->_db->describeTable($table);
-
+            $metadata = new \Zend\Db\Metadata\Metadata($this->_db);
+            $tbl = $metadata->getTable($table);
+            $columns = $tbl->getColumns();
+            $constraints = $tbl->getConstraints();
             $pkeys = array();
-            foreach ( $descr as $field ) {
-                if ( $field['PRIMARY'] == 1 ) {
-                    $pos = $field['PRIMARY_POSITION'];
-                    $pkeys[$pos] = $field['COLUMN_NAME'];
+
+            foreach ( $constraints as $constraint ) {
+                if ( $constraint->getType() === 'PRIMARY KEY' ) {
+                    $pkeys = $constraint->getColumns();
                 }
             }
 
@@ -641,14 +635,16 @@ class Db extends \Zend_Db
                 }
             }
 
-            $r = $result->fetchAll();
-            foreach ( $r as $row ) {
+            $select = $this->_sql->select($table);
+            $results = $this->execute($select);
+
+            foreach ( $results as $row ) {
                 $data = array();
                 $where = array();
 
                 //build where
                 foreach ( $pkeys as $k ) {
-                    $where[] = $k . ' = ' . $this->_db->quote($row->$k);
+                    $where[] = $k . ' = "' . $row->$k .'"';
                 }
 
                 //build data
@@ -657,11 +653,9 @@ class Db extends \Zend_Db
                 }
 
                 //finally, update data!
-                $this->_db->update(
-                    $table,
-                    $data,
-                    $where
-                );
+                $update = $this->_sql->update($table);
+                $update->set($data)->where($where);
+                $this->execute($update);
             }
         } catch (\Exception $e) {
             Analog::log(
@@ -683,18 +677,137 @@ class Db extends \Zend_Db
     }
 
     /**
-    * Global getter method
-    *
-    * @param string $name name of the variable we want to retrieve
-    *
-    * @return mixed
-    */
+     * Is current database using SQLite?
+     *
+     * @return boolean
+     */
+    public function isSQLite()
+    {
+        return $this->_type_db === self::SQLITE;
+    }
+
+    /**
+     * Instanciate a select query
+     *
+     * @param string $table Table name, without prefix
+     * @param string $alias Tables alias, optionnal
+     *
+     * @return Select
+     */
+    public function select($table, $alias = null)
+    {
+        if ( $alias === null ) {
+            return $this->_sql->select(
+                PREFIX_DB . $table
+            );
+        } else {
+            return $this->_sql->select(
+                array(
+                    $alias => PREFIX_DB . $table
+                )
+            );
+        }
+    }
+
+    /**
+     * Instanciate an insert query
+     *
+     * @param string $table Table name, without prefix
+     *
+     * @return Insert
+     */
+    public function insert($table)
+    {
+        return $this->_sql->insert(
+            PREFIX_DB . $table
+        );
+    }
+
+    /**
+     * Instanciate an update query
+     *
+     * @param string $table Table name, without prefix
+     *
+     * @return Insert
+     */
+    public function update($table)
+    {
+        return $this->_sql->update(
+            PREFIX_DB . $table
+        );
+    }
+
+    /**
+     * Instanciate a delete query
+     *
+     * @param string $table Table name, without prefix
+     *
+     * @return Delete
+     */
+    public function delete($table)
+    {
+        return $this->_sql->delete(
+            PREFIX_DB . $table
+        );
+    }
+
+    /**
+     * Execute query string
+     *
+     * @param SqlInterface $sql SQL object
+     *
+     * @return Stmt
+     */
+    public function execute($sql)
+    {
+        try {
+            $query_string = $this->_sql->getSqlStringForSqlObject($sql);
+            $this->_last_query = $query_string;
+            Analog::log(
+                'Executing query: ' . $query_string,
+                Analog::DEBUG
+            );
+            return $this->_db->query(
+                $query_string,
+                Adapter::QUERY_MODE_EXECUTE
+            );
+        } catch ( \Exception $e ) {
+            Analog::log(
+                'Query error: ' . $query_string . ' ' . $e->__toString(),
+                Analog::ERROR
+            );
+            throw $e;
+        }
+    }
+
+    /**
+     * Global getter method
+     *
+     * @param string $name name of the variable we want to retrieve
+     *
+     * @return mixed
+     */
     public function __get($name)
     {
         switch ( $name ) {
         case 'db':
             return $this->_db;
             break;
+        case 'sql':
+            return $this->_sql;
+            break;
+        case 'driver':
+            return $this->_db->getDriver();
+            break;
+        case 'connection':
+            return $this->_db->getDriver()->getConnection();
+            break;
+        case 'platform':
+            return $this->_db->getPlatform();
+            break;
+        case 'query_string':
+            return $this->_last_query;
+            break;
         case 'type_db':
             return $this->_type_db;
             break;
