@@ -7,7 +7,7 @@
  *
  * PHP version 5
  *
- * Copyright © 2006-2013 The Galette Team
+ * Copyright © 2006-2014 The Galette Team
  *
  * This file is part of Galette (http://galette.tuxfamily.org).
  *
@@ -29,7 +29,7 @@
  *
  * @author    Frédéric Jaqcuot <unknown@unknow.com>
  * @author    Johan Cwiklinski <johan@x-tnd.be>
- * @copyright 2006-2013 The Galette Team
+ * @copyright 2006-2014 The Galette Team
  * @license   http://www.gnu.org/licenses/gpl-3.0.html GPL License 3.0 or (at your option) any later version
  * @version   SVN: $Id$
  * @link      http://galette.tuxfamily.org
@@ -37,8 +37,8 @@
 
 namespace Galette\Core;
 
-use Galette\Entity\Adherent;
 use Analog\Analog as Analog;
+use Galette\Entity\Adherent;
 
 /**
  * Picture handling
@@ -48,7 +48,7 @@ use Analog\Analog as Analog;
  * @package   Galette
  * @author    Frédéric Jaqcuot <unknown@unknow.com>
  * @author    Johan Cwiklinski <johan@x-tnd.be>
- * @copyright 2006-2013 The Galette Team
+ * @copyright 2006-2014 The Galette Team
  * @license   http://www.gnu.org/licenses/gpl-3.0.html GPL License 3.0 or (at your option) any later version
  * @link      http://galette.tuxfamily.org
  */
@@ -110,10 +110,10 @@ class Picture
     protected $max_height = 200;
 
     /**
-    * Default constructor.
-    *
-    * @param int $id_adh the id of the member
-    */
+     * Default constructor.
+     *
+     * @param int $id_adh the id of the member
+     */
     public function __construct( $id_adh='' )
     {
         // '!==' needed, otherwise ''==0
@@ -141,10 +141,10 @@ class Picture
     }
 
     /**
-    * "Magic" function called on unserialize
-    *
-    * @return void
-    */
+     * "Magic" function called on unserialize
+     *
+     * @return void
+     */
     public function __wakeup()
     {
         //if file has been deleted since we store our object in the session,
@@ -167,10 +167,10 @@ class Picture
     }
 
     /**
-    * Check if current file is present on the File System
-    *
-    * @return boolean true if file is present on FS, false otherwise
-    */
+     * Check if current file is present on the File System
+     *
+     * @return boolean true if file is present on FS, false otherwise
+     */
     private function _checkFileOnFS()
     {
         $file_wo_ext = $this->store_path . $this->id;
@@ -194,18 +194,19 @@ class Picture
     }
 
     /**
-    * Check if current file is present in the database,
-    *   and copy it to the File System
-    *
-    * @return boolean true if file is present in the DB, false otherwise
-    */
+     * Check if current file is present in the database,
+     *   and copy it to the File System
+     *
+     * @return boolean true if file is present in the DB, false otherwise
+     */
     private function _checkFileInDB()
     {
         global $zdb;
 
         try {
             $select = $this->getCheckFileQuery();
-            $pic = $zdb->db->fetchRow($select);
+            $results = $zdb->execute($select);
+            $pic = $results->current();
             //what's $pic if no result?
             if ( $pic !== false ) {
                 // we must regenerate the picture file
@@ -236,32 +237,31 @@ class Picture
     }
 
     /**
-    * Returns the relevant query to check if picture exists in database.
-    *
-    * @return string SELECT query
-    */
+     * Returns the relevant query to check if picture exists in database.
+     *
+     * @return string SELECT query
+     */
     protected function getCheckFileQuery()
     {
         global $zdb;
         $class = get_class($this);
 
-        $select = new \Zend_Db_Select($zdb->db);
-        $select->from(
-            array(PREFIX_DB . $this->tbl_prefix . $class::TABLE),
+        $select = $zdb->select($this->tbl_prefix . $class::TABLE);
+        $select->columns(
             array(
                 'picture',
                 'format'
             )
         );
-        $select->where($class::PK . ' = ?', $this->db_id);
+        $select->where(array($class::PK => $this->db_id));
         return $select;
     }
 
     /**
-    * Gets the default picture to show, anyways
-    *
-    * @return void
-    */
+     * Gets the default picture to show, anyways
+     *
+     * @return void
+     */
     protected function getDefaultPicture()
     {
         $this->file_path = _CURRENT_TEMPLATE_PATH . 'images/default.png';
@@ -271,10 +271,10 @@ class Picture
     }
 
     /**
-    * Set picture sizes
-    *
-    * @return void
-    */
+     * Set picture sizes
+     *
+     * @return void
+     */
     private function _setSizes()
     {
         list($width, $height) = getimagesize($this->file_path);
@@ -299,10 +299,10 @@ class Picture
     }
 
     /**
-    * Set header and displays the picture.
-    *
-    * @return object the binary file
-    */
+     * Set header and displays the picture.
+     *
+     * @return object the binary file
+     */
     public function display()
     {
         header('Content-type: '.$this->mime);
@@ -326,14 +326,16 @@ class Picture
 
         try {
             if ( $transaction === true ) {
-                $zdb->db->beginTransaction();
+                $zdb->connection->beginTransaction();
             }
-            $del = $zdb->db->delete(
-                PREFIX_DB . $this->tbl_prefix . $class::TABLE,
-                $zdb->db->quoteInto($class::PK . ' = ?', $this->db_id)
-            );
 
-            if ( !$del > 0 ) {
+            $delete = $zdb->delete($this->tbl_prefix . $class::TABLE);
+            $delete->where(
+                $class::PK . ' = ' . $this->db_id
+            );
+            $del = $zdb->execute($delete);
+
+            if ( !$del->count() > 0 ) {
                 Analog::log(
                     'Unable to remove picture database entry for ' . $this->db_id,
                     Analog::ERROR
@@ -368,7 +370,7 @@ class Picture
             if ( $_file !== null && $success !== true ) {
                 //unable to remove file that exists!
                 if ( $transaction === true ) {
-                    $zdb->db->rollBack();
+                    $zdb->connection->rollBack();
                 }
                 Analog::log(
                     'The file ' . $_file .
@@ -378,13 +380,13 @@ class Picture
                 return false;
             } else {
                 if ( $transaction === true ) {
-                    $zdb->db->commit();
+                    $zdb->connection->commit();
                 }
                 return true;
             }
         } catch (\Exception $e) {
             if ( $transaction === true ) {
-                $zdb->db->rollBack();
+                $zdb->connection->rollBack();
             }
             Analog::log(
                 'An error occured attempting to delete picture ' . $this->db_id .
@@ -396,13 +398,13 @@ class Picture
     }
 
     /**
-    * Stores an image on the disk and in the database
-    *
-    * @param object  $file the uploaded file
-    * @param boolean $ajax If the image cames from an ajax call (dnd)
-    *
-    * @return true|false result of the storage process
-    */
+     * Stores an image on the disk and in the database
+     *
+     * @param object  $file the uploaded file
+     * @param boolean $ajax If the image cames from an ajax call (dnd)
+     *
+     * @return true|false result of the storage process
+     */
     public function store($file, $ajax = false)
     {
         /** TODO:
@@ -510,18 +512,39 @@ class Picture
         fclose($f);
 
         try {
-            $stmt = $zdb->db->prepare(
-                'INSERT INTO ' . PREFIX_DB .
-                $this->tbl_prefix . $class::TABLE . ' (' . $class::PK .
-                ', picture, format) VALUES (:id, :picture, :format)'
+            $insert = $zdb->insert($this->tbl_prefix . $class::TABLE);
+            $insert->values(
+                array(
+                    $class::PK  => ':id',
+                    'picture'   => ':picture',
+                    'format'    => ':format'
+                )
             );
+            $stmt = $zdb->sql->prepareStatementForSqlObject($insert);
+            $container = $stmt->getParameterContainer();
+            $container->offsetSet(
+                $class::PK,
+                ':id'
+            );
+            $container->offsetSet(
+                'picture',
+                ':picture',
+                $container::TYPE_LOB
+            );
+            $container->offsetSet(
+                'format',
+                ':format'
+            );
+            $stmt->setParameterContainer($container);
 
-            $stmt->bindParam('id', $this->db_id);
-            $stmt->bindParam('picture', $picture, \PDO::PARAM_LOB);
-            $stmt->bindParam('format', $extension);
-            $stmt->execute();
+            $stmt->execute(
+                array(
+                    $class::PK  => $this->db_id,
+                    'picture'   => $picture,
+                    'format'    => $extension
+                )
+            );
         } catch (\Exception $e) {
-            /** FIXME */
             Analog::log(
                 'An error occured storing picture in database: ' .
                 $e->getMessage(),
@@ -534,15 +557,15 @@ class Picture
     }
 
     /**
-    * Resize the image if it exceed max allowed sizes
-    *
-    * @param string $source the source image
-    * @param string $ext    file's extension
-    * @param string $dest   the destination image.
-    *                           If null, we'll use the source image. Defaults to null
-    *
-    * @return void
-    */
+     * Resize the image if it exceed max allowed sizes
+     *
+     * @param string $source the source image
+     * @param string $ext    file's extension
+     * @param string $dest   the destination image.
+     *                           If null, we'll use the source image. Defaults to null
+     *
+     * @return void
+     */
     private function _resizeImage($source, $ext, $dest = null)
     {
         $class = get_class($this);
@@ -641,72 +664,71 @@ class Picture
         }
     }
 
-    /* GETTERS */
     /**
-    * Returns current file optimal height (resized)
-    *
-    * @return int optimal height
-    */
+     * Returns current file optimal height (resized)
+     *
+     * @return int optimal height
+     */
     public function getOptimalHeight()
     {
         return round($this->optimal_height);
     }
 
     /**
-    * Returns current file height
-    *
-    * @return int current height
-    */
+     * Returns current file height
+     *
+     * @return int current height
+     */
     public function getHeight()
     {
         return $this->height;
     }
 
     /**
-    * Returns current file optimal width (resized)
-    *
-    * @return int optimal width
-    */
+     * Returns current file optimal width (resized)
+     *
+     * @return int optimal width
+     */
     public function getOptimalWidth()
     {
         return $this->optimal_width;
     }
 
     /**
-    * Returns current file width
-    *
-    * @return int current width
-    */
+     * Returns current file width
+     *
+     * @return int current width
+     */
     public function getWidth()
     {
         return $this->width;
     }
 
     /**
-    * Returns current file format
-    *
-    * @return string
-    */
+     * Returns current file format
+     *
+     * @return string
+     */
     public function getFormat()
     {
         return $this->format;
     }
 
     /**
-    * Have we got a picture ?
-    *
-    * @return bool True if a picture matches adherent's id, false otherwise
-    */
+     * Have we got a picture ?
+     *
+     * @return bool True if a picture matches adherent's id, false otherwise
+     */
     public function hasPicture()
     {
         return $this->has_picture;
     }
 
     /**
-    * Returns unauthorized characters litteral values quoted, comma separated values
-    *
-    * @return string comma separated disallowed characters
-    */
+     * Returns unauthorized characters litteral values quoted, comma separated values
+     *
+     * @return string comma separated disallowed characters
+     */
     public function getBadChars()
     {
         $ret = '';
@@ -717,40 +739,40 @@ class Picture
     }
 
     /**
-    * Returns allowed extensions
-    *
-    * @return string comma separated allowed extensiosn
-    */
+     * Returns allowed extensions
+     *
+     * @return string comma separated allowed extensiosn
+     */
     public function getAllowedExts()
     {
         return implode(', ', $this->_allowed_extensions);
     }
 
     /**
-    * Return the array of allowed mime types
-    *
-    * @return array
-    */
+     * Return the array of allowed mime types
+     *
+     * @return array
+     */
     public function getAllowedMimeTypes()
     {
         return $this->_allowed_mimes;
     }
 
     /**
-    * Returns current file full path
-    *
-    * @return string full file path
-    */
+     * Returns current file full path
+     *
+     * @return string full file path
+     */
     public function getPath()
     {
         return $this->file_path;
     }
 
     /**
-    * Returns current mime type
-    *
-    * @return string
-    */
+     * Returns current mime type
+     *
+     * @return string
+     */
     public function getMime()
     {
         return $this->mime;
